# Copyright (c) 2015 MaxPoint Interactive, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following
#    disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
Wrapper for avro using cython wrapped avrolib

Allows faster reading and writing of avro files.
'''

from __future__ import absolute_import, print_function
from cyavro._cyavro import AvroReader, AvroWriter, prepare_pandas_df_for_write
import pandas as pd
import os as os
import glob as glob
from filelock.filelock import FileLock
import six
import json
try:
    from version import version as __version__
except ImportError:
    pass


DEFAULT_BUFFER_SIZE = 50000
SENTINEL_INT64_NULL = -9223372036854775808
SENTINEL_INT32_NULL = -2147483648


class UnixMilliAvroReader(AvroReader):
    """Helper class to allow reading hints from avro schema for timestamp fields stored as Long

    Parameters
    ----------
    filename : str
    json_avro_schema : dict
    """

    def __init__(self, filename, json_avro_schema=None):
        super(UnixMilliAvroReader, self).__init__(filename)
        if json_avro_schema is not None:
            if isinstance(json_avro_schema, dict) and ('fields' in json_avro_schema):
                self._json_avro_schema = json_avro_schema
        self._json_avro_schema = None

    @property
    def json_avro_schema(self):
        if self._json_avro_schema is None:
            # dependency on the avro python reference implementation since getting full json
            # avro schema from the c-api is elusive
            from avro.datafile import DataFileReader
            from avro.io import DatumReader
            import json
            with open(self.filename) as fo:
                with DataFileReader(fo, DatumReader()) as avf:
                    self._json_avro_schema = json.loads(avf.meta['avro.schema'])
        return self._json_avro_schema

    def read_chunk(self):
        return_dict = super(UnixMilliAvroReader, self).read_chunk()
        # Do some postprocessing, generally dealing with dates
        for f in self.json_avro_schema['fields']:
            if f.get('sql_type', None) == 'TIMESTAMP WITH TIME ZONE':
                return_dict[f['name']] = pd.to_datetime(return_dict[f['name']], utc=True, unit='ms')
        return return_dict


# Global aliases for avro reader / writer classes.  This is to allow for user to change these at runtime to
# user defined subclasses
avro_reader_cls = AvroReader
avro_writer_cls = AvroWriter


def read_avro_file_as_dataframe_iter(path_to_file, buffer_size=DEFAULT_BUFFER_SIZE):
    """Reads an avro file.

    Parameters
    ----------
    path_to_file : string
        /path/to/the/file.avro
    buffer_size : int
        Number of rows to read per block read.

    Returns
    -------
    df : yields :class:`pd.DataFrame`
        A generator yielding :class:`pd.DataFrame` objects for each `buffer_size` chunk
    """
    reader = avro_reader_cls()
    reader.init_file(path_to_file)
    try:
        reader.init_reader()
        reader.init_buffers(buffer_size)
        while True:
            df = pd.DataFrame(reader.read_chunk())
            if len(df):
                yield df
            else:
                break
    finally:
        reader.close()


def read_avro_bytesio_as_dataframe_iter(filelikeobject, buffer_size=DEFAULT_BUFFER_SIZE):
    """Reads an avro file.

    Parameters
    ----------
    path_to_file : string
        /path/to/the/file.avro
    buffer_size : int
        Number of rows to read per block read.

    Returns
    -------
    df : yields :class:`pd.DataFrame`
        A generator yielding :class:`pd.DataFrame` objects for each `buffer_size` chunk
    """
    reader = avro_reader_cls()
    data = filelikeobject.read()
    reader.init_bytes(data)
    try:
        reader.init_reader()
        reader.init_buffers(buffer_size)
        while True:
            df = pd.DataFrame(reader.read_chunk())
            if len(df):
                yield df
            else:
                break
    finally:
        reader.close()




def group_dataframe_iter(iterator, group_by):
    """splits an iterator of dataframes into group chunks.

    This is a useful utility function in conjunction with :meth:`read_avro_path_as_dataframe_iter` as it allows a one
    pass looping over presorted chunked data with keys crossing file boundaries.

    This function will *DISCARD* the index of the dataframes.

    Parameters
    ----------
    iterator : iterator of :class:`pd.DataFrame` objects.
    group_by : first argument to pass to :method:`pd.DataFrame.groupby`

    Returns
    -------
    df : yields :class:`pd.DataFrame`
        A generator yielding :class:`pd.DataFrame` objects for each group obtained by using
        ``df.groupby(group_by, sort=False)``.  This assume that the data in the dataframes is group contiguous.

    Examples
    --------
    # Example usage and behavior
    >>> df = DataFrame({'A' : ['foo', 'bar', 'foo', ],
    ...                 'B' : ['one', 'one', 'two', ],
    ...                 'C' : range(3), 'D' : range(3)})
    ...
    >>> iterator = [df.sort('A', ascending=True), df]
    >>> for x, dff in group_avro_dataframe_iter(iterator, 'A')):
    ...     print x
    ...     print dff
    bar
         A    B  C  D
    0  bar  one  1  1
    foo
         A    B  C  D
    0  foo  one  0  0
    1  foo  two  2  2
    2  foo  one  0  0
    3  foo  two  2  2
    bar
         A    B  C  D
    0  bar  one  1  1


    """
    prev_group = None
    current_group = []
    for df in iterator:
        g = df.groupby(group_by, sort=False)
        for gk, gdf in g:
            if gk == prev_group:
                current_group.append(gdf)
            else:
                yield prev_group, pd.concat(current_group, ignore_index=True)

                # Clear the current group
                del current_group[:]
                prev_group = gk
                current_group.append(gdf)
    yield prev_group, pd.concat(current_group, ignore_index=True)


def read_avro_file_as_dataframe(path_to_file, buffer_size=DEFAULT_BUFFER_SIZE):
    """Reads an avro file.

    Parameters
    ----------
    path_to_file : string
        /path/to/the/file.avro
    buffer_size : int
        Number of rows to read per block read.

    Returns
    -------
    df : `pd.DataFrame`
        A pandas DataFrame of the avro file.
    """
    it = read_avro_file_as_dataframe_iter(path_to_file, buffer_size)
    df = pd.concat([df for df in it], ignore_index=True)
    return df


def read_avro_bytesio_as_dataframe(filelikeobject, buffer_size=DEFAULT_BUFFER_SIZE):
    it = read_avro_bytesio_as_dataframe_iter(filelikeobject, buffer_size)
    df = pd.concat([df for df in it], ignore_index=True)
    return df


def get_avro_files_in_path(path, pattern="*.avro"):
    files = glob.glob(os.path.join(path, pattern))
    return files


def read_avro_path_as_dataframe_iter(path):
    """Read all the avro files in a path

    Parameters
    ----------
    path : string
        /path/to/avro/files

    Returns
    -------
    df : yields `pd.DataFrame`
        An generator yielding a `pd.DataFrame` for each file.
    """
    files = get_avro_files_in_path(path)
    for fn in files:
        yield read_avro_file_as_dataframe(fn)


def read_avro_path_as_dataframe(path):
    """Helper function to read ALL avro files in a path a return a dataframe

    Parameters
    ----------
    path : string
        /path/to/avro/files

    Returns
    -------
    df : `pd.DataFrame`
    """
    return pd.concat(iter(read_avro_path_as_dataframe_iter(path)))


def write_avro_file_from_dataframe(df, path_to_file, schema=None, codec='snappy', block_size=1048576):
    """Writes an avro file from a given dataframe..

    Parameters
    ----------
    df : `pd.DataFrame`
        The dataframe to be written to avro
    path_to_file : string
        /path/to/the/file.avro
    schema : string or dict, optional
        The avro schema to be written.  If a schema is not supplied it will be inferred.
    codec : string
        The compression scheme required {'none', 'snappy', 'deflate'}.

    Returns
    -------
    bool
        True on completion
    """
    if schema is None:
        schema = infer_avro_schema_for_dataframe(df)
    if isinstance(schema, dict):
        schema = json.dumps(schema)
    writer = avro_writer_cls(path_to_file, str(codec), str(schema), block_size)
    try:
        writer.write(df)
    finally:
        writer.close()
    return True


class AvroToHDF(object):
    """Utility namespace class that provides methods to convert Avro to HDF5 for easier use within the
    rest of the python ecosystem.

    This implementation will chunk the avro reads and write these chunks using multiple threads.
    This will not read the entire avro file into memory at once.  This allows for safe use when converting
    large avro files to HDF5.

    Structured as a class so that consumers can subclass this if they wish to override behavior

     - :meth:`avro_file_to_hdf5`
     - :meth:`convert_df_to_hdf5`
     - :meth:`read_avro_file_as_dataframe_iter`

    Examples
    --------
    >>> class SplitByLineItemHDF(AvroToHDF)
    ...
    ...     @classmethod
    ...     def convert_df_to_hdf5(cls, df, hdf_file, table_name):
    ...         for li, subdf in df.groupby('key'):
    ...             super(SplitByLineItemHDF, cls).convert_df_to_hdf5(subdf, hdf_file, '{}_{}'.format(table_name, li))
    ...
    ... SplitByLineItemHDF.avro_path_to_hdf5('.', 'container.h5', 'key', n_jobs=4)
    """

    @classmethod
    def read_avro_file_as_dataframe_iter(cls, avro_file):
        return read_avro_file_as_dataframe_iter(avro_file)

    @classmethod
    def get_avro_files_in_path(path):
        return get_avro_files_in_path(path)

    @classmethod
    def avro_file_to_hdf5(cls, avro_file, hdf_file, table_name):
        """Converts a single avro file to a single hdf5 file.

        Parameters
        ----------
        avro_file : string
            A path to the avro file to be converted.
        hdf_file : string
            Path to the output hdf file.
        table_name : string
            Name of the table to be created in the hdf file.

        Returns
        -------
        bool
            True on completion
        """
        for df in cls.read_avro_file_as_dataframe_iter(avro_file):
                cls.convert_df_to_hdf5(df, hdf_file, table_name)
        return True

    @classmethod
    def convert_df_to_hdf5(cls, df, hdf_file, table_name):
        """Converts a given dataframe to hdf5.

        Parameters
        ----------
        df : `pd.DataFrame`
        hdf_file : string
        table_name : string
        """
        lockfile = hdf_file + '.lock'
        with FileLock(lockfile, timeout=1000000):
            df.to_hdf(hdf_file, table_name, format='table', append=True)

    @classmethod
    def avro_path_to_hdf5(cls, path, hdf_file, table_name, n_jobs=1):
        """Converts avro files to a single hdf5 file.  Optionally using joblib to run this in parallel.

        Parameters
        ----------
        path : string
            A path where the avro files to read are stored.
        hdf_file : string
            Path to the output hdf file.
        table_name : string
            Name of the table to be created in the hdf file.
        n_jobs : integer, optional
            Number of worker jobs to spawn.  If > 1 this will create `n_jobs` processes using `joblib`.
            This requires `joblib` or `sklearn` to be installed.

        Returns
        -------
        bool
            True on completion
        """
        filenames = get_avro_files_in_path(path)
        hdf_file = os.path.abspath(hdf_file)
        # Construct argument list for all jobs
        args = [{
            'avro_file': filename,
            'hdf_file': hdf_file,
            'table_name': table_name
            } for filename in filenames]

        has_joblib = False
        # Import joblib from normal location or from scikit-learn
        try:
            import joblib
            has_joblib = True
        except ImportError:
            try:
                from sklearn.externals import joblib
                has_joblib = True
            except ImportError:
                pass

        if (n_jobs > 1) and has_joblib:
            px = joblib.Parallel(n_jobs=n_jobs)
            px(joblib.delayed(cls.avro_file_to_hdf5)(**a) for a in args)
            return True
        else:
            for a in args:
                cls.avro_file_to_hdf5(**a)
        return True


# Aliases for class methods to simplify the namespace.
avro_file_to_hdf5 = AvroToHDF.avro_file_to_hdf5
avro_path_to_hdf5 = AvroToHDF.avro_path_to_hdf5


def infer_avro_schema_for_dataframe(df):
    """Get the avro json schema for a given pandas DataFrame

    Parameters
    ----------
    df : `pd.DataFrame`

    Returns
    -------
    schema : str
        Inferred avro schema as a json string.
    """
    import itertools
    import numpy as np

    def _get_python_type(iterable):
        types = set(map(type, iterable))
        if len(types) == 1:
            # Still dealing with non-union types
            tp = types.pop()
            if issubclass(tp, six.string_types):
                return '"string"'
            elif issubclass(tp, bytes):
                return '"bytes"'
            elif issubclass(tp, dict):
                # get value element type
                values = (x.itervalues() for x in iterable)
                value_type = _get_python_type(list(itertools.chain.from_iterable(values)))
                return '{{"type": "map", "values": {} }}'.format(value_type)
            elif issubclass(tp, (list, tuple)):
                value_type = _get_python_type(list(itertools.chain.from_iterable(iterable)))
                return '{{"type": "array", "items": {} }}'.format(value_type)
            elif issubclass(tp, int):
                return '"long"'
            elif issubclass(tp, float):
                return '"double"'
            elif issubclass(tp, bool):
                return '"bool"'
            elif issubclass(tp, type(None)):
                return '"null"'
            else:
                raise Exception('Unexpected type found, ' + repr(types))
        else:
            typestrings = []
            for tp in types:
                sublist = filter(lambda x: type(x) == tp, iterable)
                x = _get_python_type(sublist)
                if x == '"null"':
                    typestrings.insert(0, x)
                else:
                    typestrings.append(_get_python_type(sublist))
            if len(typestrings) > 0:
                return '[{}]'.format(','.join(typestrings))
            else:
                return '"null"'

    def _get_avro_dtype(dtype, iterable):
        if dtype == np.int64:
            return '"long"'
        elif dtype == np.int32:
            return '"int"'
        elif dtype == np.float32:
            return '"float"'
        elif dtype == np.float64:
            return '"double"'
        elif dtype == np.bool:
            return '"bool"'
        elif dtype == np.dtype('datetime64[ns]'):
            return '"long"'
        elif dtype == np.object:
            # Special handling for object types
            return _get_python_type(iterable)
        else:
            raise Exception('Unhandled dtype found, ' + repr(dtype))

    dtypes = df.dtypes
    fields = []
    for name, typ in dtypes.iteritems():
        fields.append('{"name":"%(name)s", "type": %(type)s}' % {'name': name, 'type': _get_avro_dtype(typ, df[name])})

    return '''{
    "type": "record",
    "name": "AutoGen",
    "namespace": "com.maxpoint.cyavro.autogenerated",
    "fields": [
        %(fields)s
    ]}''' % {'fields': (',\n' + ' ' * 8).join(fields)}



from _cavro cimport *

cdef enum AvroReaderType:
    avro_reader_type_unset,
    avro_reader_type_bytes,
    avro_reader_type_file

cdef class AvroReader:
    cdef avro_file_reader_t _reader
    cdef public int chunk_size
    cdef public list refholder, field_names, field_types, buffer_lst
    cdef public str filename
    cdef int initialized, empty_file, _should_free_buffer
    cdef void *fp_reader_buffer
    cdef int fp_reader_buffer_length

    cdef public bytes filedata
    cdef int filedatalength
    cdef AvroReaderType reader_type

    cdef init_reader_buffer(self)
    cdef init_file_reader(self)

    cdef init_memory_reader(self)



cdef reader_from_bytes_c(void *buffer, int length)



# Copyright (c) 2015 MaxPoint Interactive, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following
#    disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIG`HT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# This is a set of headers wrapping "avro.h" provided from libavro.
# These are required for the c api exposed by libavro to be importable in Cython.

cimport numpy as np

from libc.stdint cimport int32_t, int64_t
from libc.stdlib cimport *
from libc.stdio cimport *


cdef extern from 'avro.h':
    #'basics.h':
    cdef enum avro_type_t:
        AVRO_STRING,
        AVRO_BYTES,
        AVRO_INT32,
        AVRO_INT64,
        AVRO_FLOAT,
        AVRO_DOUBLE,
        AVRO_BOOLEAN,
        AVRO_NULL,
        AVRO_RECORD,
        AVRO_ENUM,
        AVRO_FIXED,
        AVRO_MAP,
        AVRO_ARRAY,
        AVRO_UNION,
        AVRO_LINK
    ctypedef avro_type_t avro_type_t

    cdef enum avro_class_t:
        AVRO_SCHEMA,
        AVRO_DATUM
    ctypedef avro_class_t avro_class_t

    #'schema.h':
    struct avro_obj_t:
        pass
    ctypedef avro_obj_t *avro_schema_t

    size_t avro_schema_union_size(const avro_schema_t union_schema)

    int avro_schema_decref(avro_schema_t schema)

    struct avro_schema_error_t_:
        pass
    ctypedef avro_schema_error_t_ *avro_schema_error_t

    int avro_schema_from_json(const char *jsontext, int32_t unused1,
              avro_schema_t *schema, avro_schema_error_t *unused2)

    #'value.h':
    struct avro_value_iface:
        pass
    ctypedef avro_value_iface avro_value_iface_t

    cdef struct avro_value:
        avro_value_iface_t  *iface
        void  *self
    ctypedef avro_value avro_value_t

    # Reference Counting
    void avro_value_incref(avro_value_t *value)
    void avro_value_decref(avro_value_t *value)

    # General methods
    avro_type_t avro_value_get_type(avro_value_t *value)
    avro_schema_t avro_value_get_schema(avro_value_t *value)
    int avro_value_reset(avro_value_t *value)

    # Primitive Getters

    int avro_value_get_boolean(avro_value_t *value, int *out)
    int avro_value_get_bytes(avro_value_t *value, const void **buf, size_t *size)
    int avro_value_get_double(avro_value_t *value, double *out)
    int avro_value_get_float(avro_value_t *value, float *out)
    int avro_value_get_int(avro_value_t *value, int32_t *out)
    int avro_value_get_long(avro_value_t *value, int64_t *out)
    int avro_value_get_null(avro_value_t *value)

    #''' /* The result will be NUL-terminated the size will INCLUDE the
    # * NUL terminator.  str will never be NULL unless there's an
    # * error. */'''

    int avro_get_bytes(avro_value_t *value, const void **buf, size_t *size)
    int avro_value_get_string(avro_value_t *value, const char **str, size_t *size)

    int avro_value_get_enum(avro_value_t *value, int *out)
    int avro_value_get_fixed(avro_value_t *value, const void **buf, size_t *size)

    # Compound Getters
    int avro_value_get_size(avro_value_t *value, size_t *size)
    int avro_value_get_by_index(avro_value_t *value, size_t index, avro_value_t *child, const char **name)
    int avro_value_get_by_name(avro_value_t *value, const char *name, avro_value_t *child, size_t *index)

    # Union Types
    int avro_value_get_discriminant(avro_value_t *value, int *out)
    int avro_value_get_current_branch(avro_value_t *value, avro_value_t *branch)

    ## Setters
    int avro_value_set_boolean(avro_value_t *value, int val)
    int avro_value_set_double(avro_value_t *value, double val)
    int avro_value_set_float(avro_value_t *value, float val)
    int avro_value_set_int(avro_value_t *value, int32_t val)
    int avro_value_set_long(avro_value_t *value, int64_t val)
    int avro_value_set_null(avro_value_t *value)
    int avro_value_set_bytes(avro_value_t *value, void *buf, size_t size)
    int avro_value_set_fixed(avro_value_t *value, void *buf, size_t size)
    int avro_value_set_string_len(avro_value_t *value, const char *str, size_t size)
    int avro_value_set_enum(const avro_value_t *value, int val)

    # Compound value setters
    int avro_value_append(avro_value_t *value, avro_value_t *child_out, size_t *new_index)
    int avro_value_add(avro_value_t *value, const char *key, avro_value_t *child, size_t *new_index, int *is_new)

    # union types
    int avro_value_set_branch(avro_value_t *value, int discriminant, avro_value_t *branch)

    #'generic.h':

    avro_value_iface_t * avro_generic_class_from_schema(avro_schema_t schema)

    #/**
    #* Allocate a new instance of the given generic value class.  @a iface
    #* must have been created by @ref avro_generic_class_from_schema.
    #*/

    int avro_generic_value_new(avro_value_iface_t *iface, avro_value_t *dest)

    void avro_value_iface_decref(avro_value_iface_t *iface)
    void avro_value_iface_incref(avro_value_iface_t *iface)

    #"io.h":
    # Struct infer from header
    struct avro_reader_t_:
        pass
    ctypedef avro_reader_t_ *avro_reader_t

    struct avro_writer_t_:
        pass
    ctypedef avro_writer_t_ *avro_writer_t

    struct avro_file_reader_t_:
        pass
    ctypedef avro_file_reader_t_ *avro_file_reader_t

    struct avro_file_writer_t_:
        pass
    ctypedef avro_file_writer_t_ *avro_file_writer_t

    int avro_file_writer_create_with_codec(const char *path,
                avro_schema_t schema, avro_file_writer_t * writer,
                const char *codec, size_t block_size)

    int avro_file_writer_create(const char *path, avro_schema_t schema,
                avro_file_writer_t * writer)

    int avro_file_reader(const char *path, avro_file_reader_t * reader)
    int avro_file_reader_fp(FILE *fp, const char *path, int should_close, avro_file_reader_t * reader)

    avro_schema_t avro_file_reader_get_writer_schema(avro_file_reader_t reader)

    int avro_file_writer_sync(avro_file_writer_t writer)
    int avro_file_writer_flush(avro_file_writer_t writer)
    int avro_file_writer_close(avro_file_writer_t writer)

    int avro_file_reader_close(avro_file_reader_t reader)

    int avro_file_reader_read_value(avro_file_reader_t reader, avro_value_t *dest)
    int avro_file_writer_append_value(avro_file_writer_t writer, avro_value_t *src)

    void avro_reader_reset(avro_reader_t reader)
    void avro_writer_reset(avro_writer_t writer)

    #error handling
    const char* avro_strerror()



# Copyright (c) 2015 MaxPoint Interactive, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following
#    disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from distutils.core import setup
from Cython.Build import cythonize
from distutils.extension import Extension
import numpy as np
import os

#import Cython.Compiler.Options
#Cython.Compiler.Options.annotate = True

prefix = os.getenv('PREFIX', None)
if prefix is not None:
    include_dirs = [os.path.join(prefix, 'include')]
    library_dirs = [os.path.join(prefix, 'lib')]
else:
    include_dirs = []
    library_dirs = []

include_dirs.append(np.get_include())

extensions = [
    Extension(
        'cyavro._cyavro', ['cyavro/*.pyx'],
        include_dirs=include_dirs,
        library_dirs=library_dirs,
        libraries=['avro', 'm', 'snappy'],
    )
]

version = str(os.environ.get('PKG_VERSION', "0.6.0"))


def write_version_py():
    content = """\
version = '%s'
""" % version

    filename = os.path.join(os.path.dirname(__file__), 'cyavro', 'version.py')
    with open(filename, 'w') as fo:
        fo.write(content)

write_version_py()


setup(name='cyavro',
      version=version,
      packages=['cyavro'],
      package_data={'cyavro': ['_cyavro.c', '*.pyx', '*.pxd']},
      description='Wrapper to avro-c-1.7.5',
      maintainer='MaxPoint Interactive',
      maintainer_email='marius.vanniekerk@maxpoint.com',
      author='MaxPoint Interactive',
      author_email='marius.vanniekerk@maxpoint.com',
      requires=['pandas (>=0.12)',
                'numpy (>=1.7.0)',
                'cython (>=0.19.1)'],
      ext_modules=cythonize(extensions, cython_gdb=True),
      url='https://github.com/maxpoint/cyavro',
      license='BSD',
      )
